@@ -501,7 +501,7 @@ sub test_serializers : Tests {
         sub {
             $self->new_cache( serializer => [1] );
         },
-        qr/Validation failed for 'Serializer'/,
+        qr/Validation failed for/,
         "invalid serializer"
     );
     lives_ok(
@@ -656,7 +656,7 @@ sub test_multi_no_keys : Test(4) {
     lives_ok { $cache->remove_multi( [] ) } "remove_multi (no args)";
 }
 
-sub test_l1_cache : Test(228) {
+sub test_l1_cache : Test(238) {
     my $self   = shift;
     my @keys   = map { "key$_" } ( 0 .. 2 );
     my @values = map { "value$_" } ( 0 .. 2 );
@@ -718,7 +718,7 @@ sub test_l1_cache : Test(228) {
 
         $self->_test_logging_with_l1_cache( $cache, $l1_cache );
 
-        $self->_test_common_subcache_features( $cache, $l1_cache );
+        $self->_test_common_subcache_features( $cache, $l1_cache, 'l1_cache' );
     };
 
     # Test with current cache in primary position...
@@ -743,7 +743,7 @@ sub test_l1_cache : Test(228) {
     $test_l1_cache->();
 }
 
-sub test_mirror_cache : Test(206) {
+sub test_mirror_cache : Test(216) {
     my $self = shift;
     my ( $cache, $mirror_cache );
     my ( $key, $value, $key2, $value2 ) = $self->kvpair(2);
@@ -767,7 +767,8 @@ sub test_mirror_cache : Test(206) {
 
         $self->_test_logging_with_mirror_cache( $cache, $mirror_cache );
 
-        $self->_test_common_subcache_features( $cache, $mirror_cache );
+        $self->_test_common_subcache_features( $cache, $mirror_cache,
+            'mirror_cache' );
     };
 
     my $file_cache_options = sub {
@@ -900,20 +901,25 @@ sub _test_logging_with_mirror_cache {
 # Run tests common to l1_cache and mirror_cache
 #
 sub _test_common_subcache_features {
-    my ( $self, $cache, $subcache ) = @_;
-    my ( $key, $value, $key2, $value2 ) = $self->kvpair(2);
+    my ( $self, $cache, $subcache, $subcache_type ) = @_;
+    my ( $key,  $value, $key2,     $value2 )        = $self->kvpair(2);
 
     for ( $cache, $subcache ) { $_->clear() }
 
     # Test informational methods
     #
-    ok( !$cache->is_subcache,             "is_subcache - false" );
-    ok( $subcache->is_subcache,           "is_subcache - true" );
-    ok( !defined( $cache->parent_cache ), "parent_cache - undef" );
+    ok( !$cache->is_subcache,         "is_subcache - false" );
+    ok( $subcache->is_subcache,       "is_subcache - true" );
+    ok( $cache->has_subcaches,        "has_subcaches - true" );
+    ok( !$subcache->has_subcaches,    "has_subcaches - false" );
+    ok( !$cache->can('parent_cache'), "parent_cache - cannot" );
     is( $subcache->parent_cache, $cache, "parent_cache - defined" );
-    ok( !defined( $cache->subcache_type ), "subcache_type - undef" );
-    cmp_deeply( $cache->subcaches,    [$subcache], "subcaches - empty" );
-    cmp_deeply( $subcache->subcaches, [],          "subcaches - empty" );
+    ok( !$cache->can('subcache_type'), "subcache_type - cannot" );
+    is( $subcache->subcache_type, $subcache_type, "subcache_type - defined" );
+    cmp_deeply( $cache->subcaches, [$subcache], "subcaches - defined" );
+    ok( !$subcache->can('subcaches'), "subcaches - cannot" );
+    is( $cache->$subcache_type, $subcache, "$subcache_type - defined" );
+    ok( !$subcache->can($subcache_type), "$subcache_type - cannot" );
 
     # Test that sets and various kinds of removals and expirations are distributed to both
     # the primary cache and the subcache
@@ -1085,11 +1091,16 @@ sub test_cache_object : Test(6) {
     );
 }
 
-sub test_size_awareness : Test(9) {
+sub test_size_awareness : Test(11) {
     my $self = shift;
     my ( $key, $value ) = $self->kvpair();
 
-    ok( !$self->{cache}->is_size_aware(), "not size aware by default" );
+    ok( !$self->new_cleared_cache()->is_size_aware(),
+        "not size aware by default" );
+    ok( $self->new_cleared_cache( is_size_aware => 1 )->is_size_aware(),
+        "is_size_aware turns on size awareness" );
+    ok( $self->new_cleared_cache( max_size => 10 )->is_size_aware(),
+        "max_size turns on size awareness" );
 
     my $cache = $self->new_cleared_cache( is_size_aware => 1 );
     is( $cache->get_size(), 0, "size is 0 for empty" );
