@@ -26,9 +26,9 @@ sub is_bad {
 
 sub test_driver_options : Test(7) {
     my $cache;
-    is_good( driver       => 'Memory' );
+    is_good( driver       => 'Memory',              global => 1 );
     is_good( driver       => 'File' );
-    is_good( driver_class => 'CHI::Driver::Memory' );
+    is_good( driver_class => 'CHI::Driver::Memory', global => 1 );
     is_good( driver_class => 'CHI::Driver::File' );
     is_bad( driver_class => 'Memory' );
     is_bad( driver       => 'CHI::Driver::File' );
